@@ -1,26 +1,11 @@
-'''
-[LICENSE]
-Copyright (c) 2020 Alliance for Sustainable Energy, LLC, All Rights Reserved
-
-Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
-
-1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
-
-2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
-
-3. The entire corresponding source code of any redistribution, with or without modification, by a research entity, including but not limited to any contracting manager/operator of a United States National Laboratory, any institution of higher learning, and any non-profit organization, must be made publicly available under this license for as long as the redistribution is made available by the research entity.
-
-4. Redistribution of this software, without modification, must refer to the software by the same designation. Redistribution of a modified version of this software (i) may not refer to the modified version by the same designation, or by any confusingly similar designation, and (ii) must refer to the underlying software originally provided by Alliance as "layerstack". Except to comply with the foregoing, the term "layerstack", or any confusingly similar designation may not be used to refer to any modified version of this software or any modified version of the underlying software originally provided by Alliance without the prior written consent of Alliance.
-
-5. The name of the copyright holder(s), any contributors, the United States Government, the United States Department of Energy, or any of their employees may not be used to endorse or promote products derived from this software without specific prior written permission from the respective party.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER(S) AND ANY CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER(S), ANY CONTRIBUTORS, THE UNITED STATES GOVERNMENT, OR THE UNITED STATES DEPARTMENT OF ENERGY, NOR ANY OF THEIR EMPLOYEES, BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-[/LICENSE]
-'''
 """
+layerstack.args
+---------------
 Classes that allow the specification of arguments that can be serialized to and
 loaded from json, as well as expressed on the command line.
+
+:copyright: (c) 2021, Alliance for Sustainable Energy, LLC
+:license: BSD-3
 """
 
 from collections import OrderedDict
@@ -75,7 +60,10 @@ class KwArgBase(object):
     def nargs(self, value):
         if value not in [None, '?', '+', '*']:
             self._nargs = int(value)
-            self._is_list = True
+            self._is_list = True # following argparse convention that if 
+                                 # nargs = 1, the single element is placed in a
+                                 # list
+            return
         self._nargs = value
         self._is_list = self._nargs in ['+', '*']
 
@@ -97,6 +85,11 @@ class KwArgBase(object):
         """
         return self._is_list
 
+    def get_name(self, cleaned=True):
+        if cleaned:
+            return self.name.replace("-", "_")
+        return self.name
+
     def _set_value(self, value):
         """
         Called by derived classes to set their _value attribute.
@@ -120,9 +113,19 @@ class KwArgBase(object):
             values = value
             if self.parser is not None:
                 values = [self.parser(value) for value in values]
-            return self.list_parser(values) if self.list_parser is not None else values
+            if self.list_parser is not None:
+                values = self.list_parser(values)
+            if self.choices is not None:
+                if not values in self.choices:
+                    for value in values:
+                        if value not in self.choices:
+                            raise LayerStackRuntimeError(f"{values} and {value} not in choices = {self.choices}")
+            return values
         assert not self.is_list
-        return self.parser(value) if self.parser is not None else value
+        value = self.parser(value) if self.parser is not None else value
+        if (self.choices is not None) and (value not in self.choices):
+            raise LayerStackRuntimeError(f"{value} not in the allowable choices: {self.choices}")
+        return value
 
     def add_argument_kwargs(self):
         """
@@ -178,6 +181,15 @@ class Arg(KwArgBase):
                          save_parser=save_parser)
         self.name = name
 
+    def __repr__(self):
+        return (f"{self.__class__.__name__}({self.name!r},\n"
+            f"    description = {self.description!r},\n"
+            f"    parser = {self.parser!r},\n"
+            f"    choices = {self.choices!r},\n"
+            f"    nargs = {self.nargs!r},\n"
+            f"    list_parser = {self.list_parser!r},\n"
+            f"    save_parser = {self.save_parser!r})")
+
     @property
     def set(self):
         """
@@ -252,6 +264,15 @@ class Kwarg(KwArgBase):
         self.action = action
         self.default = self._process_value(default,none_allowed=True)
 
+    def __repr__(self):
+        return (f"{self.__class__.__name__}(default = {self.default!r},\n"
+            f"      description = {self.description!r},\n"
+            f"      parser = {self.parser!r},\n"
+            f"      choices = {self.choices!r},\n"
+            f"      nargs = {self.nargs!r},\n"
+            f"      list_parser = {self.list_parser!r},\n"
+            f"      save_parser = {self.save_parser!r})")
+
     @property
     def defaulted(self):
         """
@@ -474,15 +495,16 @@ class ArgList(list):
             unexpected behavior.
         """
         if not self.mode == ArgMode.USE:
-            raise LayerStackRuntimeError("{} ".format(self.__class__.__name__) + 
+            raise LayerStackRuntimeError(f"{self.__class__.__name__} "
                 "must be in ArgMode.USE to set values.")
         try:
             self.mode = ArgMode.DESC
             for arg in self:
                 try:
-                    temp_value = eval('cli_args.' + arg.name) 
+                    temp_value = getattr(cli_args, arg.get_name(cleaned=False))
                 except Exception as e:
-                    raise LayerStackRuntimeError("{} not found in cli_args, {}".format(arg.name,e))
+                    raise LayerStackRuntimeError(f"{arg.get_name(cleaned=False)} not found in "
+                        f"cli_args:\n{cli_args},\nbecause {e}")
                 arg.value = temp_value # may throw if temp_value is bad per arg.parser, etc.
             self.mode = ArgMode.USE
         except Exception as e:
@@ -590,7 +612,7 @@ class KwargDict(OrderedDict):
             (if mode == ArgMode.DESC), or its .value (if mode == ArgMode.USE)
         """
         if self.mode == ArgMode.USE:
-            return [(name, kwarg.value) for name, kwarg in super().items()]
+            return [(kwarg.get_name(), kwarg.value) for kwarg in super().values()]
         return super().items()
 
     def iteritems(self):
@@ -605,7 +627,7 @@ class KwargDict(OrderedDict):
             ArgMode.USE)
         """
         for name, kwarg in super().items():
-            yield (name, kwarg) if self.mode == ArgMode.DESC else (name, kwarg.value)
+            yield (name, kwarg) if self.mode == ArgMode.DESC else (kwarg.get_name(), kwarg.value)
 
     def values(self):
         """
@@ -695,30 +717,8 @@ class KwargDict(OrderedDict):
             raise LayerStackRuntimeError("{} ".format(self.__class__.__name__) + 
                 "must be in ArgMode.DESC to add arguments to an argparse parser.")
 
-        def get_short_name(name):
-            short_name = None; n = 0
-
-            # first try splitting on '_' to get good letters
-            n = 0
-            chars = "".join([x[0] for x in name.split('_')])
-            while not short_name:
-                n += 1
-                if chars[:n] not in short_names:
-                    short_name = chars[:n]
-
-            # now just use all characters
-            n = 1
-            while not short_name:
-                n += 1
-                if name[:n] not in short_names:
-                    short_name = name[:n]
-            
-            assert short_name
-            short_names.append(short_name)
-            return short_name
-
         for name, kwarg in self.items():
-            short_name = get_short_name(name)
+            short_name = get_short_name(name, short_names=short_names)
             kwarg_kwargs = kwarg.add_argument_kwargs()
             try:
                 parser.add_argument('-' + short_name, '--' + name,
@@ -759,12 +759,67 @@ class KwargDict(OrderedDict):
             self.mode = ArgMode.DESC
             for key, kwarg in self.items():
                 try:
-                    temp_value = eval('cli_args.' + key) 
+                    temp_value = getattr(cli_args, kwarg.get_name())
                 except Exception as e:
-                    raise LayerStackRuntimeError("{} not found in cli_args, {}".format(key,e))
+                    raise LayerStackRuntimeError(f"{kwarg.get_name()} not found in "
+                        f"cli_args:\n{cli_args},\nbecause {e}")
                 kwarg.value = temp_value # may throw if temp_value is bad per kwarg.parser, etc.
             self.mode = ArgMode.USE
         except Exception as e:
             self.mode = ArgMode.USE
             raise e
-        
\ No newline at end of file
+        
+
+def get_short_name(name, short_names = [], seps = ['_', '-']):
+    short_name = None
+
+    def multisplit(astr, seps):
+        parts = None
+        for sep in seps:
+            if parts is None:
+                parts = astr.split(sep)
+                continue
+            tmp = []
+            for part in parts:
+                tmp.extend(part.split(sep))
+            parts = tmp
+        return parts
+
+    parts = multisplit(name, seps)
+    len_parts = [len(part) for part in parts]
+
+    M = 1; N = len(parts); candidates = set(); k = 0
+    logger.debug(parts)
+    while not short_name:
+        if k and (len(candidates) == k):
+            raise LayerStackRuntimeError("Unable to find a short name "
+                "for {name}. Current short names:\n[\n  {short_names}\n]"
+                "\nStarted with parts = {parts},\nevaluated "
+                "candidates:\n[\n  {candidates}\n]".format(
+                    name = name, 
+                    short_names = ",\n  ".join([repr(sn) for sn in short_names]),
+                    parts = parts,
+                    candidates = ",\n  ".join([repr(candi) for candi in candidates])
+                ))
+        n = 1; k = len(candidates)
+        logger.debug(f"M = {M}; N = {N}; k = {k}")
+        while n <= N:
+            candidate = ''
+            for i in range(n):
+                m = min(len_parts[i], M)
+                candidate += parts[i][:m]
+                logger.debug(f"  n = {n}; i = {i}; m = {m}; {candidate}")
+            for i in range(n,N):
+                mm1 = min(len_parts[i], M - 1)
+                candidate += parts[i][:mm1]
+                logger.debug(f"  n = {n}; N = {N}; i = {i}; mm1 = {mm1}; {candidate}")
+            if candidate not in short_names:
+                short_name = candidate
+                break
+            candidates.add(candidate)
+            n += 1
+        M += 1
+    
+    assert short_name
+    short_names.append(short_name)
+    return short_name
