@@ -1,22 +1,6 @@
 '''
-[LICENSE]
-Copyright (c) 2020 Alliance for Sustainable Energy, LLC, All Rights Reserved
-
-Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
-
-1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
-
-2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
-
-3. The entire corresponding source code of any redistribution, with or without modification, by a research entity, including but not limited to any contracting manager/operator of a United States National Laboratory, any institution of higher learning, and any non-profit organization, must be made publicly available under this license for as long as the redistribution is made available by the research entity.
-
-4. Redistribution of this software, without modification, must refer to the software by the same designation. Redistribution of a modified version of this software (i) may not refer to the modified version by the same designation, or by any confusingly similar designation, and (ii) must refer to the underlying software originally provided by Alliance as "layerstack". Except to comply with the foregoing, the term "layerstack", or any confusingly similar designation may not be used to refer to any modified version of this software or any modified version of the underlying software originally provided by Alliance without the prior written consent of Alliance.
-
-5. The name of the copyright holder(s), any contributors, the United States Government, the United States Department of Energy, or any of their employees may not be used to endorse or promote products derived from this software without specific prior written permission from the respective party.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER(S) AND ANY CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER(S), ANY CONTRIBUTORS, THE UNITED STATES GOVERNMENT, OR THE UNITED STATES DEPARTMENT OF ENERGY, NOR ANY OF THEIR EMPLOYEES, BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-[/LICENSE]
+:copyright: (c) 2021, Alliance for Sustainable Energy, LLC
+:license: BSD-3
 '''
 
 from __future__ import print_function, division, absolute_import
@@ -133,7 +117,7 @@ class LayerBase(object):
         arg_list = cls.args()
         arg_list.add_arguments(parser)
         kwarg_dict = cls.kwargs()
-        kwarg_dict.add_arguments(parser, short_names=['r', 'd'])
+        kwarg_dict.add_arguments(parser, short_names=['r', 'd', 'h'])
 
         # Parse args and set values        
         cli_args = parser.parse_args()
@@ -199,7 +183,6 @@ class LayerBase(object):
         assert arg_list.mode == ArgMode.USE
         assert kwarg_dict.mode == ArgMode.USE
         from layerstack.stack import Stack
-        # TODO: Fix KwargDict so **kwarg_dict works natively
         return cls.apply(Stack(run_dir=cli_args.run_dir), 
             *arg_list, **{k: v for k, v in kwarg_dict.items()})
 
@@ -416,7 +399,7 @@ class Layer(object):
         Directory from which to load the layer
     """
 
-    def __init__(self, layer_dir):
+    def __init__(self, layer_dir, model=None):
         self.layer_dir = layer_dir
         # load the layer.py module and find the LayerBase class
         # self._layer = the LayerBase class we found
@@ -424,9 +407,16 @@ class Layer(object):
         self._layer = self.load_layer(layer_dir)
         self._checksum = checksum(self.layer_filename(layer_dir))
         self._name = self._layer.name
-        self._args = self._layer.args()
+        
+        if issubclass(self._layer, ModelLayerBase):
+            logger.debug(f"Using {model} to populate args and kwargs")
+            self._args = self._layer.args(model = model)
+            self._kwargs = self._layer.kwargs(model = model)
+        else:
+            self._args = self._layer.args()
+            self._kwargs = self._layer.kwargs()
+
         self._args.mode = ArgMode.DESC
-        self._kwargs = self._layer.kwargs()
         self._kwargs.mode = ArgMode.DESC
 
     @classmethod
@@ -436,13 +426,13 @@ class Layer(object):
 
         Parameters
         ----------
-        name : 'str'
+        name : str
             Layer name
-        parent_dir : 'str'
+        parent_dir : str or pathlib.Path
             Parent directory for layer
-        desc : 'str'
+        desc : str
             Layer description
-        layer_base_class : 'LayerBase|ModelLayerBase'
+        layer_base_class : LayerBase or child class
             Base class on which to build layer
 
         Returns
@@ -452,6 +442,7 @@ class Layer(object):
         """
 
         # Create the directory
+        parent_dir = Path(parent_dir)
         if not parent_dir.exists():
             raise LayerStackError(f"The parent_dir {parent_dir} does not exist.") # maynot need the msg_begin here
         dir_name = name.lower().replace(" ", "_")
@@ -514,7 +505,11 @@ class Layer(object):
         def class_doc_str(aclass, doc_str=''):
             if issubclass(aclass, LayerBase):
                 result = '\n    {}\n    {}'.format(aclass.__name__, '=' * len(aclass.__name__))
-                result += aclass.__doc__
+                if aclass.__doc__ is not None:
+                    result += aclass.__doc__
+                else:
+                    logger.info("You may consider writing a doc string for "
+                        f"LayerBase derived class {aclass.__name__}")
 
                 for base_class in aclass.__bases__:
                     result = class_doc_str(base_class, doc_str=(result + doc_str))
@@ -578,6 +573,7 @@ class Layer(object):
 
         module = load_module_from_file('loaded_layer_{}'.format(uuid4()),
                                        Layer.layer_filename(layer_dir))
+
         candidate = None
         base_classes = [LayerBase, ModelLayerBase]
         for item in dir(module):
@@ -690,6 +686,14 @@ hierarchy tree.".format(layer_dir)
         for name, value in kwargs.items():
             self._kwargs[name] = value
 
+    def set_arg_mode(self, arg_mode):
+        """
+        Convenience function for setting args and kwargs objects to the same
+        ArgMode.
+        """
+        self.args.mode = arg_mode
+        self.kwargs.mode = arg_mode
+
     @classmethod
     def run(cls, layer_dir, stack, model, *args, **kwargs):
         """
