@@ -1,38 +1,24 @@
 '''
-[LICENSE]
-Copyright (c) 2020 Alliance for Sustainable Energy, LLC, All Rights Reserved
-
-Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
-
-1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
-
-2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
-
-3. The entire corresponding source code of any redistribution, with or without modification, by a research entity, including but not limited to any contracting manager/operator of a United States National Laboratory, any institution of higher learning, and any non-profit organization, must be made publicly available under this license for as long as the redistribution is made available by the research entity.
-
-4. Redistribution of this software, without modification, must refer to the software by the same designation. Redistribution of a modified version of this software (i) may not refer to the modified version by the same designation, or by any confusingly similar designation, and (ii) must refer to the underlying software originally provided by Alliance as "layerstack". Except to comply with the foregoing, the term "layerstack", or any confusingly similar designation may not be used to refer to any modified version of this software or any modified version of the underlying software originally provided by Alliance without the prior written consent of Alliance.
-
-5. The name of the copyright holder(s), any contributors, the United States Government, the United States Department of Energy, or any of their employees may not be used to endorse or promote products derived from this software without specific prior written permission from the respective party.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER(S) AND ANY CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER(S), ANY CONTRIBUTORS, THE UNITED STATES GOVERNMENT, OR THE UNITED STATES DEPARTMENT OF ENERGY, NOR ANY OF THEIR EMPLOYEES, BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-[/LICENSE]
-Contents
---------
+layerstack.stack
+----------------
 - :class:`layerstack.stack.Stack`
 - :func:`layerstack.stack.main`
 
+:copyright: (c) 2021, Alliance for Sustainable Energy, LLC
+:license: BSD-3
 '''
 from __future__ import print_function, division, absolute_import
 
 import argparse
-from collections import MutableSequence, OrderedDict
+from collections import OrderedDict
+from collections.abc import MutableSequence
 import json
 import logging
 from pathlib import Path
 from os import chdir
 from timeit import default_timer as timer
 from uuid import UUID, uuid4
+import sys
 
 logger = logging.getLogger(__name__)
 
@@ -312,8 +298,7 @@ class Stack(MutableSequence):
         ArgMode.
         """
         for layer in self.layers:
-            layer.args.mode = arg_mode
-            layer.kwargs.mode = arg_mode
+            layer.set_arg_mode(arg_mode)
 
     def save(self, filename):
         """
@@ -695,7 +680,7 @@ class Stack(MutableSequence):
             end_file_log(logfile)
         except:
             chdir(old_cur_dir)
-            logger.info(f"Stack failed after {timer_str(timer() - start)}")
+            logger.error(f"Stack failed after {timer_str(timer() - start)}")
             end_file_log(logfile)
             raise        
 
@@ -747,7 +732,7 @@ def repoint_stack(p, layer_library_dir=None, original_locations_preferred=True,
     return    
 
 
-def main():
+def parse_args_helper(args):
     parser = argparse.ArgumentParser("Load and optionally run a stack.")
     
     # all CLI options require loading a Stack json file
@@ -789,7 +774,12 @@ def main():
         turn off stack archiving.""", dest='archive', action='store_false')
     parser_run.set_defaults(archive=True)
 
-    args = parser.parse_args()
+    return parser.parse_args(args)
+
+
+def main():
+
+    args = parse_args_helper(sys.argv[1:])
 
     # determine log level
     log_level = logging.INFO
@@ -824,7 +814,7 @@ def main():
         assert False, f'Unknown mode {args.mode}'
     
     return
-
+    
 
 if __name__ == '__main__':
     main()
